#
#  MyDocument.rb
#  RubyRaiseMan
# 


require 'osx/cocoa'
require 'Person.rb'
require 'PreferenceController.rb'

class MyDocument < OSX::NSDocument
  
  ib_outlets :person_controller, :table_view
  
  kvc_array_accessor :employees # this is needed for key-value-observing
  
  attr_reader :employees
  	
  def init
    super_init
    @employees = Array.new
    
    notification_center = OSX::NSNotificationCenter.defaultCenter
    notification_center.addObserver_selector_name_object(self, :handle_color_changed, PreferenceController::RRMTableBgColorChanged, nil)
    self
  end
  
  def handle_color_changed(notification)
    # The notification object is of type PreferenceController
    # because of this line: notification_center.postNotification(notification)
    table_color = notification.userInfo.objectForKey(PreferenceController::RRMTableBgColorKey)
    @table_view.setBackgroundColor(table_color)
  end
  
  def employees=(array)
    if @employees == array
      return
    end
    
    # we need to stop observing old ones before assigning to array
    @employees.each {|p| self.stopObservingPerson(p)}
    
    @employees = array
    
    # start observing the new data
    @employees.each {|p| self.startObservingPerson(p)}
  end
  
  def remove(sender)
    selected = @person_controller.selectedObjects
    choice = OSX::NSRunAlertPanel("Delete", "Do you want to delete %@ records?", "Delete", "Cancel", "Zero raise", selected.size)
    
    if  choice == OSX::NSAlertDefaultReturn
      @person_controller.remove(sender)
    elsif choice == OSX::NSAlertOtherReturn
      # there might be a better way to do this
      @person_controller.selectedObjects.each do |p|
        p.expected_raise = 0.0
      end
    end
  end
  
  def insertObject_inEmployeesAtIndex(person, index)
    # OSX::NSLog("insertObject called")
    undo_manager = self.undoManager
    (undo_manager.prepareWithInvocationTarget(self)).removeObjectFromEmployeesAtIndex(index)
    
    unless undo_manager.isUndoing?
      undo_manager.setActionName("Insert Person")
    end
    
    self.startObservingPerson(person)
    @employees.insert(index, person)
  end
  
  def removeObjectFromEmployeesAtIndex(index)
    # OSX::NSLog("removeObject called")
    person = @employees[index]
    undo_manager = self.undoManager
    undo_manager.prepareWithInvocationTarget(self).insertObject_inEmployeesAtIndex(person, index)
    
    unless undo_manager.isUndoing?  
      undo_manager.setActionName("Delete Person")
    end
    
    self.stopObservingPerson(person)
    @employees.delete_at(index)
  end
  
  def startObservingPerson(person)
    person.addObserver_forKeyPath_options_context(self, "person_name", OSX::NSKeyValueObservingOptionOld, nil)
    person.addObserver_forKeyPath_options_context(self, "expected_raise", OSX::NSKeyValueObservingOptionOld, nil)
  end
  
  def stopObservingPerson(person)
    person.removeObserver_forKeyPath(self, "person_name")
    person.removeObserver_forKeyPath(self, "expected_raise")    
  end
  
  def changeKeyPath_ofObject_toValue(keyPath, obj, newValue)
    obj.setValue_forKeyPath(newValue, keyPath)
  end
  
  def observeValueForKeyPath_ofObject_change_context(keyPath, obj, change, context)
    undo_manager = self.undoManager
    oldValue = change.objectForKey(OSX::NSKeyValueChangeOldKey)
    (undo_manager.prepareWithInvocationTarget(self)).changeKeyPath_ofObject_toValue(keyPath, obj, oldValue)
    undo_manager.setActionName("Edit")
  end
  
  def windowNibName
    return "MyDocument" # this is the name of our NIB file
  end

  def windowControllerDidLoadNib (aController)
    super_windowControllerDidLoadNib(aController)
    defaults = OSX::NSUserDefaults.standardUserDefaults
    color_as_data = defaults.objectForKey(PreferenceController::RRMTableBgColorKey)
    @table_view.setBackgroundColor(OSX::NSKeyedUnarchiver.unarchiveObjectWithData(color_as_data))
  end

  # The two functions below will save the file and load the file
  def dataRepresentationOfType (aType)
    @person_controller.commitEditing
    return OSX::NSKeyedArchiver.archivedDataWithRootObject(employees)
  end

  def loadDataRepresentation_ofType (data, aType)
    # OSX::NSLog("Reading data") 
    employees = OSX::NSKeyedUnarchiver.unarchiveObjectWithData(data)
    @employees = employees.to_a
    return true
  end

  def revertDocumentToSaved(sender)
    OSX::NSLog("Revert")
  end
end
